#!/usr/bin/perl -w

###
# Pod Documentation
###

=head1 NAME

index_db.pl

=head1 SYNOPSIS

index_db.pl [-f genbank -d database_name] input_directory output_directory

Create a database index for all sequence files in the input directory.

=head1 DESCRIPTION

=over

=item B<-d> <database name> 

Name of the database index to be created.

=item B<-f> <format> 

Format of the database files. May be "genbank", "fasta", or "embl". The default value is "genbank".

=head1 NOTES 

If you move the sequence files, you will need to re-create the database index. 

=back

=head1 AUTHOR

Jeffrey Barrick

=head1 COPYRIGHT

Copyright (C) 2008 Jeffrey Barrick.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

###
# End Pod Documentation
###

use strict;

use FindBin;
use lib $FindBin::Bin;

use Bio::DB::Flat;
use Data::Dumper;
use Getopt::Long;
use Pod::Usage;

my ($help, $man);
my ($format, $db_name);

pod2usage(1) if (scalar @ARGV == 0);
GetOptions(
	'help|?' => \$help, 'man' => \$man,
	'format|f=s' => \$format,
	'database-name|d=s' => \$db_name,
) or pod2usage(2);
my ($input_dir, $output_dir) = @ARGV;
pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;
pod2usage(1) if (!defined $input_dir || !defined $output_dir || !defined $db_name);

$format = 'genbank' if (!defined $format);	
my $in_file_list_ref = _dir_file_list_to_file_list($input_dir);
my $param = { format => $format, directory => $output_dir, dbname => $db_name, file_list => \@$in_file_list_ref };

mkdir "$output_dir" if (!-e $output_dir);
CreateIndex($param);


sub CreateIndex
{
	my ($param) = @_;
	#print Dumper($param);
		
	my $Options = { -directory => $param->{directory}, -dbname => $param->{dbname}, -index => 'flat', -write_flag => 1, -format => $param->{format} };
	
	#Double escape patterns.
	if ($param->{format} eq 'fasta')
	{ 
		$Options->{-start_pattern} = q/^>/;
		$Options->{-primary_pattern} = q/^>.+?\|.+?\|.+?\|(.+?)\|/;
		$Options->{-primary_namespace} = "ID";
	}
	elsif ($param->{format} eq 'genbank')
	{
		$Options->{-start_pattern} = "^LOCUS";
		$Options->{-primary_pattern} = q/^VERSION\s+(\S+)/;
		$Options->{-primary_namespace} = "ID";
	}
	elsif ($param->{format} eq 'embl')
	{
		$Options->{-start_pattern} = "^ID";
		$Options->{-primary_pattern} = q/^SV\s+(\S+)/;
		$Options->{-primary_namespace} = "ID";
	}
	else
	{
		die "Unrecognized format: $param->{format}";
	}
	
	#print Dumper($Options);
	
	my $db = Bio::DB::Flat->new(%$Options) or die "can't create Bio::DB::Flat object";
   
   	#Must set this manually...
    $db->alphabet('nucleotide');
    
    my $entries = $db->build_index(@{$param->{file_list}});

    print STDERR "Indexed $entries entries.\n ";
}

sub _dir_file_list_to_file_list
{
	my @file_list;
		
	foreach my $file_or_dir (@_)
	{
		if (-d $file_or_dir)
		{
			opendir(INDIR, $file_or_dir) or die "Cannot open directory: $file_or_dir";
			my @dir_files = grep !/^\./, readdir INDIR;
			@dir_files = grep !/^_/, @dir_files;
			
			foreach my $file ( @dir_files)
			{
				push @file_list, "$file_or_dir/$file" if (! -d "$file_or_dir/$file");
			}
		}
		else
		{
			push @file_list, $file_or_dir;
		}
	}
	return \@file_list; 
}