#!/usr/bin/perl -w


###
# Pod Documentation
###

=head1 NAME

genomic_context.pl

=head1 SYNOPSIS

genomic_context.pl [--svg -b bit_scores.tab -c cddid.tbl] [ -d database_index_path OR -s sequence.gbk ] input.sto output_prefix 

Draw pictures of the genomic context of each hit in the input Stockholm file 
in PNG or SVG format.

=head1 DESCRIPTION

=over

=item B<input.sto> <file path> 

Stockholm format RNA alignment file of the sequences whose genomic contexts will be found.

=item B<output_prefix> <base path> 

Prefix for alignment files.

=item B<database_index_path> <directory path> 

Path containing the index files for the database of GenBank records to be used to create genomic contexts. Ex: /tutorial/db/index

=item B<database_name> <string> 

Name of the index files that correspond to the database to be used to create genomic contexts. Ex: refseq24

=item B<cdd_info> <file path> 

Path to the cddid.tbl text file for the CDD database.

=item B<--svg> <path> 

Create pictures in SVG format, rather than PNG format.

=item B<-a|--print-accession-position>

Add a line like NC_000913.2/23453-23150 to HTML output.

=item B<-t|--print_taxonomy>

Add an NCBI taxonomy line to HTML output.

=back

=head1 AUTHOR

Jeffrey Barrick

=head1 COPYRIGHT

Copyright (C) 2008 Jeffrey Barrick.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

###
# End Pod Documentation
###

use Getopt::Std;
use Data::Dumper;

use FindBin;
use lib $FindBin::Bin;
use Bio::SeqIO;
use Bio::AlignIO;
use GenomeContext;
use GenomeContextDrawer;
use CddDescriptions;

use Getopt::Long;
use Pod::Usage;

use CGI qw/:standard *table *Tr/; ;
my $q = new CGI;


my ($help, $man);
my $format = 'png';
my ($cdd_info_name, $bit_score_file, $sequence_file, $db_path, $print_accession_position, $print_taxonomy);
pod2usage(1) if (scalar @ARGV == 0);
GetOptions(
	'help|?' => \$help, 'man' => \$man,
	'format|f=s' => \$format,
	'bit-scores|b=s' => \$bit_score_file,
	'cdd-info|c=s' => \$cdd_info_name,
	'sequence-file|s=s' => \$sequence_file,
	'database-path|d=s' => \$db_path,
	'print-accession-position|a' => \$print_accession_position,
	'print-taxonomy|t' => \$print_taxonomy,
) or pod2usage(2);
$format = "\U$format";
die "Unrecognized drawing format: $format\n" if ( ($format ne 'PNG') && ($format ne 'SVG') );

if ($sequence_file && $db_path)
{
	print "The -s and -g options may not be used simultaneously.\n";
	pod2usage(1);
}

my ($input_file, $prefix) = @ARGV;
pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;
pod2usage(1) if (!defined $input_file or !defined $prefix);
pod2usage(1) if (!defined $db_path && !defined $sequence_file);

my $seq_hash = { };
if ($sequence_file)
{
	my $in = Bio::SeqIO->new(-file => "<$sequence_file");
	while (my $seq = $in->next_seq)
	{
		my $key =  $seq->accession . "." . $seq->version;
		$seq_hash->{$key} = $seq;
	}
}

my $in = Bio::AlignIO->new(-format => 'stockholm', -file   => $input_file);
my $cdd_desc;
my $cdd_desc_file_name = $FindBin::Bin . "/cddid.tbl";
$cdd_desc = CddDescriptions->new(-file_name => $cdd_desc_file_name ) if (-e $cdd_desc_file_name);

my %bit_scores;
if (defined $bit_score_file)
{
	open BIT_SCORES, "<$bit_score_file" or die "Could not open $bit_score_file";
	while (<BIT_SCORES>)
	{
		chomp $_;
		my @sl = split /\t/, $_;
		$bit_scores{$sl[0]} = $sl[1];
	}	 
}

#Debugging
#print Dumper(%bit_scores);


my @gcs;
my $aln = $in->next_aln();
my $i=1;
foreach my $seq ( $aln->each_seq() )
{
	my $accession_position = $seq->display_id . "/" . $seq->start . "-" . $seq->end;
	$seq->strand(+1);
	if ($seq->start > $seq->end)
	{
		my $temp = $seq->start;
		$seq->start($seq->end);
		$seq->end($temp);
		$seq->strand(-1);
	}
	#Goes with my version of AlignIO::stockholm
	
	#my ($start, $end) = ($seq->start, $seq->end);
	#($start, $end) = ($end, $start) if ($seq->strand == -1);
	#$accession_position .= "/$start-$end";
	
	#print Dumper($seq);
	
	#print $seq->strand() . "\n";
	
	print "$i: $accession_position\n";
	
	my %options = (
		-upstream => 2000, 
		-downstream => 8000,
		-accession => $seq->display_name,
		-start => $seq->start,
		-end => $seq->end,
		-strand => $seq->strand,
	);
	
	if ($sequence_file)
	{
		$options{-seq_hash} = $seq_hash;
	}
	else
	{
		$options{-db_path} = $db_path;
		$options{-record_cache_path} = "$db_path/cache";
	}
	
	my $gc = GenomeContext->new_neighborhood(%options);
	
	
	$gc->{accession_position} = $accession_position;
		
	#print Dumper($gc);	
	push @gcs, $gc;
	$gc->{i} = $i;
	$i++;
	
#	last if ($i == 100); #For testing
}

print "Drawing...\n";

my $gc_drawer = GenomeContextDrawer->new(-svg => ($format eq 'SVG'), -cdd_desc => $cdd_desc);
$gc_drawer->draw(out_path => $prefix, regions => \@gcs);

open INDEX, ">$prefix/index.html";
print INDEX start_html("$input_file Genomic Context");
foreach my $gc (@gcs)
{
	
	my $header =  p . "Match #" . $gc->{i};
	$header .= " Bit Score = " . $bit_scores{$gc->{accession_position}} if (defined $bit_scores{$gc->{accession_position}});
	$header .= br . ">$gc->{accession_position}" if ($print_accession_position);
	$header .= br . $gc->{taxonomy} if ($print_taxonomy && defined $gc->{taxonomy});
	$header .= br;

	print INDEX span({style => "font-family: courier; font-size: 10pt"}, $header);

	if ($format eq 'PNG')
	{
		print INDEX img({-src=>"$gc->{i}.\L$format", -border=>1 });
	}
	elsif ($format eq 'SVG')
	{
		print INDEX "<embed src=\"$gc->{i}.\L$format\" border=1 width=$gc_drawer->{img_width} height=$gc_drawer->{img_height} pluginspage=\"http://www.adobe.com/svg/viewer/install/\">";
	}
}
print INDEX end_html;
close INDEX;